@@ -47,7 +47,7 @@ use std::{
     fmt,
 };
 
-use spinning_top::{Spinlock, SpinlockGuard};
+use spin1::{Mutex as Spinlock, MutexGuard as SpinlockGuard};
 use crate::signal::{Signal, SyncSignal};
 
 /// An error that may be emitted when attempting to send a value into a channel on a sender when
@@ -113,6 +113,14 @@ impl<T> fmt::Display for TrySendError<T> {
 
 impl<T> std::error::Error for TrySendError<T> {}
 
+impl<T> From<SendError<T>> for TrySendError<T> {
+    fn from(err: SendError<T>) -> Self {
+        match err {
+            SendError(item) => Self::Disconnected(item),
+        }
+    }
+}
+
 /// An error that may be emitted when sending a value into a channel on a sender with a timeout when
 /// the send operation times out or all receivers are dropped.
 #[derive(Copy, Clone, PartialEq, Eq)]
@@ -149,6 +157,14 @@ impl<T> fmt::Display for SendTimeoutError<T> {
 
 impl<T> std::error::Error for SendTimeoutError<T> {}
 
+impl<T> From<SendError<T>> for SendTimeoutError<T> {
+    fn from(err: SendError<T>) -> Self {
+        match err {
+            SendError(item) => Self::Disconnected(item),
+        }
+    }
+}
+
 enum TrySendTimeoutError<T> {
     Full(T),
     Disconnected(T),
@@ -195,6 +211,14 @@ impl fmt::Display for TryRecvError {
 
 impl std::error::Error for TryRecvError {}
 
+impl From<RecvError> for TryRecvError {
+    fn from(err: RecvError) -> Self {
+        match err {
+            RecvError::Disconnected => Self::Disconnected,
+        }
+    }
+}
+
 /// An error that may be emitted when attempting to wait for a value on a receiver with a timeout
 /// when the receive operation times out or all senders are dropped and there are no values left
 /// in the channel.
@@ -217,6 +241,14 @@ impl fmt::Display for RecvTimeoutError {
 
 impl std::error::Error for RecvTimeoutError {}
 
+impl From<RecvError> for RecvTimeoutError {
+    fn from(err: RecvError) -> Self {
+        match err {
+            RecvError::Disconnected => Self::Disconnected,
+        }
+    }
+}
+
 enum TryRecvTimeoutError {
     Empty,
     Timeout,
@@ -328,8 +360,8 @@ impl<T> Hook<T, SyncSignal> {
     }
 }
 
+#[cfg(feature = "spin")]
 #[inline]
-#[cfg(not(windows))]
 fn wait_lock<T>(lock: &Spinlock<T>) -> SpinlockGuard<T> {
     let mut i = 4;
     loop {
@@ -345,18 +377,18 @@ fn wait_lock<T>(lock: &Spinlock<T>) -> SpinlockGuard<T> {
     }
 }
 
+#[cfg(not(feature = "spin"))]
 #[inline]
-#[cfg(windows)]
 fn wait_lock<'a, T>(lock: &'a Mutex<T>) -> MutexGuard<'a, T> {
     lock.lock().unwrap()
 }
 
-#[cfg(windows)]
+#[cfg(not(feature = "spin"))]
 use std::sync::{Mutex, MutexGuard};
 
-#[cfg(not(windows))]
+#[cfg(feature = "spin")]
 type ChanLock<T> = Spinlock<T>;
-#[cfg(windows)]
+#[cfg(not(feature = "spin"))]
 type ChanLock<T> = Mutex<T>;
 
 
@@ -582,7 +614,7 @@ impl<T> Shared<T> {
     /// msgs that have already been sent)
     fn disconnect_all(&self) {
         self.disconnected.store(true, Ordering::Relaxed);
-    
+
         let mut chan = wait_lock(&self.chan);
         chan.pull_pending(false);
         if let Some((_, sending)) = chan.sending.as_ref() {
@@ -729,7 +761,7 @@ pub struct Receiver<T> {
 
 impl<T> Receiver<T> {
     /// Attempt to fetch an incoming value from the channel associated with this receiver,
-    /// returning an error if the channel is empty.
+    /// returning an error if the channel is empty or if all senders have been dropped.
     pub fn try_recv(&self) -> Result<T, TryRecvError> {
         self.shared.recv_sync(None).map_err(|err| match err {
             TryRecvTimeoutError::Disconnected => TryRecvError::Disconnected,
